package com.ruoyi.postCheck.controller;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.postCheck.domain.PostProgrammeTemplate;
import com.ruoyi.postCheck.service.IPostProgrammeTemplateService;
import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 岗检方案模板Controller
 *
 * @author csg
 * @date 2023-12-19
 */
@Api(tags = "2.1.2-岗检方案模板接口")
@RestController
@RequestMapping("/postCheck/programmeTemplate")
public class PostProgrammeTemplateController extends BaseController {
    @Autowired
    private IPostProgrammeTemplateService postProgrammeTemplateService;

    /**
     * 查询岗检方案模板列表
     */
    @ApiOperation("查询岗检方案模板列表")
//    @PreAuthorize("@ss.hasPermi('postCheck:programmeTemplate:list')")
    @GetMapping("/list")
    public TableDataInfo list(PostProgrammeTemplate postProgrammeTemplate) {
        startPage();
        List<PostProgrammeTemplate> list = postProgrammeTemplateService.selectPostProgrammeTemplateList(postProgrammeTemplate);
        return getDataTable(list);
    }

    /**
     * 导出岗检方案模板列表
     @ApiOperation("导出岗检方案模板列表")
     @PreAuthorize("@ss.hasPermi('postCheck:programmeTemplate:export')")
     @Log(title = "岗检方案模板", businessType = BusinessType.EXPORT)
     @PostMapping("/export") public void export(HttpServletResponse response, PostProgrammeTemplate postProgrammeTemplate)
     {
     List<PostProgrammeTemplate> list = postProgrammeTemplateService.selectPostProgrammeTemplateList(postProgrammeTemplate);
     ExcelUtil<PostProgrammeTemplate> util = new ExcelUtil<PostProgrammeTemplate>(PostProgrammeTemplate.class);
     util.exportExcel(response, list, "岗检方案模板数据");
     }
     */

    /**
     * 获取岗检方案模板详细信息
     */
    @ApiOperation("获取岗检方案模板详细信息")
//    @PreAuthorize("@ss.hasPermi('postCheck:programmeTemplate:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(
            @ApiParam(name = "id", value = "岗检方案模版id", required = true)
            @PathVariable("id") Long id) {
        return success(postProgrammeTemplateService.selectPostProgrammeTemplateById(id));
    }

    /**
     * 新增岗检方案模板
     */
    @ApiOperation("新增岗检方案模板")
/*    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "主键", dataType = "Long", dataTypeClass = Long.class),
            @ApiImplicitParam(name = "planName", value = "方案名称", dataType = "String", dataTypeClass = String.class),
            @ApiImplicitParam(name = "templateName", value = "模板名称", dataType = "String", dataTypeClass = String.class),
            @ApiImplicitParam(name = "diskPath", value = "模板磁盘路径", dataType = "String", dataTypeClass = String.class),
            @ApiImplicitParam(name = "status", value = "状态（1-启动；2-作废）", dataType = "Integer", dataTypeClass = Integer.class),
            @ApiImplicitParam(name = "delFlag", value = "del_flag", dataType = "Integer", dataTypeClass = Integer.class),
            @ApiImplicitParam(name = "createUserId", value = "创建者用户ID", dataType = "Long", dataTypeClass = Long.class),
    })*/
    @PreAuthorize("@ss.hasPermi('postCheck:programmeTemplate:add')")
    @Log(title = "岗检方案模板", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody PostProgrammeTemplate postProgrammeTemplate) {
        return toAjax(postProgrammeTemplateService.insertPostProgrammeTemplate(postProgrammeTemplate));
    }

    /**
     * 修改岗检方案模板
     */
    @ApiOperation("修改岗检方案模板")
    @PreAuthorize("@ss.hasPermi('postCheck:programmeTemplate:edit')")
    @Log(title = "岗检方案模板", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody PostProgrammeTemplate postProgrammeTemplate) {
        return toAjax(postProgrammeTemplateService.updatePostProgrammeTemplate(postProgrammeTemplate));
    }

    /**
     * 删除岗检方案模板
     */
    @ApiOperation("删除岗检方案模板")
    @PreAuthorize("@ss.hasPermi('postCheck:programmeTemplate:remove')")
    @Log(title = "岗检方案模板", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(
            @ApiParam(name = "ids", value = "岗检方案模板ids", required = true)
            @PathVariable Long[] ids) {
        return toAjax(postProgrammeTemplateService.deletePostProgrammeTemplateByIds(ids));
    }


    @ApiOperation("岗检方案模板启用/禁用")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "主键ID", dataType = "Long", dataTypeClass = Long.class),
            @ApiImplicitParam(name = "status", value = "状态（1-启动；2-作废）", dataType = "Long", dataTypeClass = Long.class)
    })
    @Log(title = "岗检方案模板启用/禁用", businessType = BusinessType.UPDATE)
    @PutMapping("/approve")
    public AjaxResult approveHr(Long id, Integer status) {

        int updateCount = postProgrammeTemplateService.approve(id, status);

        return toAjax(updateCount);
    }

}
