package com.ruoyi.postCheck.domain;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.fasterxml.jackson.annotation.JsonFormat;
import org.apache.commons.lang3.builder.ToStringBuilder;
import org.apache.commons.lang3.builder.ToStringStyle;
import com.ruoyi.common.annotation.Excel;
import com.ruoyi.common.core.domain.BaseEntity;
import com.baomidou.mybatisplus.annotation.TableField;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import com.baomidou.mybatisplus.annotation.TableName;
import lombok.Data;
import lombok.EqualsAndHashCode;

/**
 * 岗检表管理对象 post_checked
 *
 * @author csg
 * @date 2024-01-05
 */
@EqualsAndHashCode(callSuper = true)
@Data
@ApiModel(value = "岗检表管理对象")
@TableName("post_checked") // 数据库表名
public class PostChecked extends BaseEntity {
    private static final long serialVersionUID = 1L;

    private Long id;

    @ApiModelProperty("岗检编号")
    @TableField(value = "post_code")
    @Excel(name = "岗检编号")
    private String postCode;

    @ApiModelProperty("检查工作组ID")
    @TableField(value = "check_workgroup_id")
    @Excel(name = "检查工作组ID")
    private Long checkWorkgroupId;

    @ApiModelProperty("检查小组ID")
    @TableField(value = "check_group_id")
    @Excel(name = "检查小组ID")
    private Long checkGroupId;

    @ApiModelProperty("被检查部门ID")
    @TableField(value = "checked_dept_id")
    @Excel(name = "被检查部门ID")
    private Long checkedDeptId;

    @ApiModelProperty("被检查岗位ID")
    @TableField(value = "checked_post_id")
    @Excel(name = "被检查岗位ID")
    private Long checkedPostId;

    @ApiModelProperty("打印人ID")
    @TableField(value = "print_user_id")
    @Excel(name = "打印人ID")
    private Long printUserId;

    @ApiModelProperty("打印时间")
    @TableField(value = "print_time")
    @Excel(name = "打印时间", width = 30, dateFormat = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date printTime;

    @ApiModelProperty("上传：检查后的检查表照片或PDF，磁盘相对路径")
    @TableField(value = "checked_img")
    @Excel(name = "上传：检查后的检查表照片或PDF，磁盘相对路径")
    private String checkedImg;

    @ApiModelProperty("上传：检查人中文姓名")
    @TableField(value = "checked_user_name")
    @Excel(name = "上传：检查人中文姓名", readConverterExp = "上=传页面设置的值")
    private String checkedUserName;

    @ApiModelProperty("上传：检查时间")
    @TableField(value = "checked_time")
    @JsonFormat(pattern = "yyyy-MM-dd")
    @Excel(name = "上传：检查时间", readConverterExp = "上=传页面设置的值")
    private Date checkedTime;

    @ApiModelProperty("上传：检查表上传人ID")
    @TableField(value = "upload_user_id")
    @Excel(name = "上传：检查表上传人ID")
    private Long uploadUserId;

    @ApiModelProperty("上传：检查表上传时间")
    @TableField(value = "upload_time")
    @Excel(name = "上传：检查表上传时间", width = 30, dateFormat = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date uploadTime;

    @ApiModelProperty("创建者用户ID")
    @TableField(value = "create_user_id")
    @Excel(name = "创建者用户ID")
    private Long createUserId;

    @ApiModelProperty("检查表明细")
    @TableField(exist = false)
    private List<PostCheckedItems> children = new ArrayList<>();


    //=======显示值============
    @ApiModelProperty("检查工作组名称")
    @TableField(exist = false)
    private String checkWorkgroupName;

    @ApiModelProperty("检查小组名称")
    @TableField(exist = false)
    private String checkGroupName;

    @ApiModelProperty("被检查部门属性")
    @TableField(exist = false)
    private String checkedDeptTypeView;

    @ApiModelProperty("受检部门类型-Version-部门类型：1-职能；2-生产")
    @TableField(value = "checked_dept_type")
    private Integer checkedDeptType;

    @ApiModelProperty("受检部门名称")
    @TableField(value = "checked_dept_name")
    private String checkedDeptName;

    @ApiModelProperty("受检岗位名称")
    @TableField(value = "checked_post_name")
    private String checkedPostName;


    @ApiModelProperty("状态：1-待审核；3-通过；4-驳回；")
    @TableField(value = "status")
    @Excel(name = "状态：1-待审核；3-通过；4-驳回；")
    private Integer status;

    @ApiModelProperty("审核人ID")
    @TableField(value = "status_by_user_id")
    @Excel(name = "审核人ID")
    private Long statusByUserId;

    @ApiModelProperty("审核人姓名")
    @TableField(value = "status_by_user_name")
    @Excel(name = "审核人姓名")
    private String statusByUserName;

    @ApiModelProperty("审核不通过原因")
    @TableField(value = "reason")
    @Excel(name = "审核不通过原因")
    private String reason;

    @ApiModelProperty("审核时间")
    @TableField(value = "status_approve_time")
    @Excel(name = "审核时间")
    private Date statusApproveTime;

    @ApiModelProperty("检查类型：1-全要素；2-自定义")
    @TableField(value = "checked_type")
    private Integer checkedType;

    @ApiModelProperty("版本号已岗位为单位累加，例如：2023-1、2023-2、…… 清单审核完成时自动设置版本号")
    @TableField("version_no")
    private String versionNo;


    @ApiModelProperty("岗检表ID集合")
    @TableField(exist = false)
    private List<Long> idList;

    @ApiModelProperty("被检查部门ID集合")
    @TableField(exist = false)
    private List<Long> deptIdList;

    /**
     * 集合状态：0-待提交；1-待审核；3-通过；4-驳回；
     */
    @TableField(exist = false)
    private List<Integer> statusList;
}
