package com.ruoyi.system.mapper;

import java.util.List;

import com.ruoyi.system.domain.SysPost;
import com.ruoyi.system.domain.UserPostAuthority;
import org.apache.ibatis.annotations.Param;

/**
 * 岗位信息 数据层
 *
 * @author ruoyi
 */
public interface SysPostMapper {
    /**
     * 查询岗位数据集合
     *
     * @param post 岗位信息
     * @return 岗位数据集合
     */
    public List<SysPost> selectPostList(SysPost post);

    /**
     * 查询所有岗位
     *
     * @return 岗位列表
     */
    public List<SysPost> selectPostAll(@Param("deptId") Long deptId);

    /**
     * 通过岗位ID查询岗位信息
     *
     * @param postId 岗位ID
     * @return 角色对象信息
     */
    public SysPost selectPostById(Long postId);

    /**
     * 根据用户ID获取岗位选择框列表
     *
     * @param userId 用户ID
     * @return 选中岗位ID列表
     */
    public List<Long> selectPostListByUserId(Long userId);

    /**
     * 查询用户所属岗位组
     *
     * @param userName 用户名
     * @return 结果
     */
    public List<SysPost> selectPostsByUserName(String userName);

    /**
     * 删除岗位信息
     *
     * @param postId 岗位ID
     * @return 结果
     */
    public int deletePostById(Long postId);

    /**
     * 批量删除岗位信息
     *
     * @param postIds 需要删除的岗位ID
     * @return 结果
     */
    public int deletePostByIds(Long[] postIds);

    /**
     * 修改岗位信息
     *
     * @param post 岗位信息
     * @return 结果
     */
    public int updatePost(SysPost post);

    /**
     * 新增岗位信息
     *
     * @param post 岗位信息
     * @return 结果
     */
    public int insertPost(SysPost post);

    /**
     * 校验岗位名称
     *
     * @param postName 岗位名称
     * @return 结果
     */
    public SysPost checkPostNameUnique(@Param("postName") String postName,@Param("deptId") Long deptId);

    /**
     * 校验岗位编码
     *
     * @param postCode 岗位编码
     * @return 结果
     */
    public SysPost checkPostCodeUnique(String postCode);

    List<UserPostAuthority> userPostAuthorityList(@Param("deptId") Long deptId, @Param("postLevel") Integer postLevel);

    List<UserPostAuthority> userHaveAuthorityList(@Param("deptCode") String deptCode, @Param("postLevel") Integer postLevel);

    /**
     * 根据部门ID集合查询下面的岗位列表
     *
     * @param deptIds
     * @return
     */
    List<SysPost> selectPostListByDeptIds(@Param("deptIds") List<Long> deptIds);

    /**
     * 查询全部的岗位和归属部门信息
     *
     * @param sysPost
     * @return
     */
    List<SysPost> selectPostDeptList(SysPost sysPost);

    /**
     * 批量插入岗位
     *
     * @param addSysPostList
     */
    void batchInsertPost(List<SysPost> addSysPostList);


    /**
     * 岗位下人员计数
     * @param postId
     * @return
     */
    int countUserByPost(@Param("postId") Long postId);

    /**
     * 查询岗位
     *
     * @param deptId
     * @param postName
     * @return
     */
    List<SysPost> selectByNameAndDeptId(@Param("deptId") Long deptId, @Param("postName") String postName);
}
